<?php

namespace Flat101\PopupAddtocart\Controller\Cart;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product\Image\UrlBuilder;
use Magento\Checkout\Model\Cart as CustomerCart;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Form\FormKey\Validator;
use Magento\Framework\Json\Helper\Data;
use Magento\Framework\Pricing\Helper\Data as PriceHelper;
use Magento\Store\Model\StoreManagerInterface;

class Add extends \Magento\Checkout\Controller\Cart\Add
{
    /**
     * @var UrlBuilder
     */
    private $_urlBuilder;
    /**
     * @var PriceHelper
     */
    private $_priceHelper;

    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        Session $checkoutSession,
        StoreManagerInterface $storeManager,
        Validator $formKeyValidator,
        CustomerCart $cart,
        ProductRepositoryInterface $productRepository,
        UrlBuilder $urlBuilder,
        PriceHelper $priceHelper
    ) {
        parent::__construct($context, $scopeConfig, $checkoutSession, $storeManager, $formKeyValidator, $cart, $productRepository);

        $this->_urlBuilder = $urlBuilder;
        $this->_priceHelper = $priceHelper;
    }

    /**
     * Resolve response
     *
     * @param string $backUrl
     * @param \Magento\Catalog\Model\Product $product
     * @return ResponseInterface|ResultInterface
     */
    protected function goBack($backUrl = null, $product = null)
    {
        if (!$this->getRequest()->isAjax()) {
            return parent::_goBack($backUrl);
        }

        $result = [];

        if ($backUrl || $backUrl = $this->getBackUrl()) {
            $result['backUrl'] = $backUrl;
        } else {
            if ($product && !$product->getIsSalable()) {
                $result['product'] = [
                    'statusText' => __('Out of stock')
                ];
            }
        }

        if ($product && $product->getIsSalable()) {
            //clear the message queue
            $this->messageManager->getMessages(true);

            $attributes = [];
            $productID = null;
            $attributeId = null;
            $params = $this->getRequest()->getParams();
            $configurableOptions = $product->getExtensionAttributes()->getConfigurableProductOptions();
            $images = $product->getMediaGalleryImages();
            $price = $product->getFinalPrice();
            $quantity = isset($params['qty']) ? $params['qty'] : 1;
            $finalPrice = $this->_priceHelper->currency($quantity * $price, true, false);

            $imageUrl = $this->_urlBuilder->getUrl($images->getFirstItem()->getFile(), 'product_page_image_medium');

            if (isset($params['super_attribute'])) {
                $selectedAttributes = $params['super_attribute'];
                foreach ($configurableOptions as $attribute) {
                    $attributeId = $attribute->getAttributeId();
                    $superAttributeId = $attribute->getProductSuperAttributeId();
                    $value = $selectedAttributes[$attributeId];
                    $productID = $value;
                    $option = $attribute->getOptionsMap($superAttributeId . ':' . $value)['store_label'];
                    $attributes[] = $option;
                }
            }

            $result['product'] = [
                'title' => $product->getName(),
                'price' => $finalPrice,
                'image' => $imageUrl,
                'qty' => $quantity,
                'attributes' => $attributes,
                'id' => $productID,
                'attrId' => $attributeId,
            ];
        }

        $this->getResponse()->representJson(
            $this->_objectManager->get(Data::class)->jsonEncode($result)
        );

        return $this->getResponse();
    }
}
